﻿// CLIENT'S INTERFACE FOR DELIVERY & TRANSPORTATION MANAGEMENT SYSTEM (CIFDTMS)
// Author: Alexander A. Stepanov (alexander.a.stepanov@gmail.com)

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Net;
using System.IO;
using System.Diagnostics;

namespace CIFDTMS{
    
    [ComVisible(true)]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    [ProgId("CIFDTMS")]

    public class Access {
        [ComVisible(true)]
        
        public string GetDistanceDurationByAddress(
            string Point1, string Point2,
            int MapServicesOption, string MapServicesKey,
            string MapServicesLang, bool Traffic) {
        
            string 
                Lang, Key, Traffic_Model, xURL,
                Str, _Str1, _Str2,
                Distance, Duration,
                Latitude, Longitude;
            int NumPos1, NumPos2;

            string[] Text = new string[100];
            Text[1] = "ERROR 01";   // Google Maps, 'OVER_QUERY_LIMIT', 'You have exceeded your rate-limit for this API.';
            Text[2] = "ERROR 02";   // Google Maps, 'INVALID_REQUEST', 'The request includes an invalid parameter, or a required parameter is missing.';
            Text[3] = "ERROR 03";   // Google Maps, 'REQUEST_DENIED', 'The provided API key is invalid.';

            Text[21] = "ERROR 21";   // Bing Maps, 'Not Found', 'One or more locations specified in the waypoint parameter are invalid or require more information.';

            switch (MapServicesLang)
            {
                case "ENU": Lang = "en-us"; break;
                case "RUS": Lang = "ru-ru"; break;
                default: Lang = "en-us"; break;
            };
            
            if (MapServicesKey != "") Key = "&key=" + MapServicesKey;
            else Key = "";

            Traffic_Model = "";
            if (Traffic)
            {
                switch (MapServicesOption)
                {
                case 0: Traffic_Model = "&departure_time=now&traffic_model=best_guess"; break;
                case 1: Traffic_Model = "&optmz=timeWithTraffic"; break;
                }
            };

            xURL = "";
            switch (MapServicesOption) {
                case 0: // "Google Maps"
                    xURL = "https://maps.googleapis.com/maps/api/distancematrix/xml?origins=" +
                           Point1 + "&destinations=" + Point2 +
                           "&mode=car&units=metric&language=" +
                           Lang + Traffic_Model + Key;
                    break;
                case 1: // "Microsoft Bing Maps"
                    xURL = "http://dev.virtualearth.net/REST/V1/Routes?o=xml&wp.0=" +
                           Point1 + "&wp.1=" + Point2 +
                           "&travelMode=Driving&ra=routeSummariesOnly&distanceUnit=km&c=" +
                           Lang + Traffic_Model + Key;
                    break;
            };

            WebRequest request = WebRequest.Create(xURL);
            ((HttpWebRequest)request).UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)";
            WebResponse response = request.GetResponse();
            Stream data = response.GetResponseStream();
            StreamReader reader = new StreamReader(data);
            Str = reader.ReadToEnd();
            reader.Close();
            response.Close();

            Console.WriteLine(Str);

            switch (MapServicesOption) {
                case 0: //"Google Maps"
                    if (Str.IndexOf("OVER_QUERY_LIMIT") > 0) return Text[1] + ": " + Point1 + "->" + Point2;
                    if (Str.IndexOf("INVALID_REQUEST") > 0) return Text[2] + ": " + Point1 + "->" + Point2;
                    if (Str.IndexOf("REQUEST_DENIED") > 0) return Text[3] + ": " + Point1 + "->" + Point2;
                    
                    NumPos1 = Str.IndexOf("<distance>") + 10;
                    _Str1 = Str.Substring(NumPos1);
                    NumPos1 = _Str1.IndexOf("<value>") + 7;
                    NumPos2 = _Str1.IndexOf("</value>");
                    _Str1 = Convert.ToString(Convert.ToDouble(_Str1.Substring(NumPos1, NumPos2 - NumPos1)) / 1000);

                    if (!Traffic) NumPos1 = Str.IndexOf("<duration>") + 10;
                    else NumPos1 = Str.IndexOf("<duration_in_traffic>") + 21;
                    _Str2 = Str.Substring(NumPos1);
                    NumPos1 = _Str2.IndexOf("<value>") + 7;
                    NumPos2 = _Str2.IndexOf("</value>");
                    _Str2 = _Str2.Substring(NumPos1, NumPos2 - NumPos1);
                    break;
                case 1: // "Microsoft Bing Maps"
                    if (Str.IndexOf("Not Found") > 0) return Text[21] + ":" + Point1 + "->" + Point2;
                    NumPos1 = Str.IndexOf("<TravelDistance>") + 16;
                    NumPos2 = Str.IndexOf("</TravelDistance>");
                    _Str1 = Str.Substring(NumPos1, NumPos2 - NumPos1);
                    _Str1 = _Str1.Replace('.', ',');

                    if (!Traffic)
                    {
                        NumPos1 = Str.IndexOf("<TravelDuration>") + 16;
                        NumPos2 = Str.IndexOf("</TravelDuration>");
                    }
                    else
                    {
                        NumPos1 = Str.IndexOf("<TravelDurationTraffic>") + 23;
                        NumPos2 = Str.IndexOf("</TravelDurationTraffic"); 
                    }
                    _Str2 = Str.Substring(NumPos1, NumPos2 - NumPos1);
                    break;
                default:
                    _Str1 = "";
                    _Str2 = "";
                    break;
            }

            Distance = _Str1;
            Duration = _Str2;
            return Distance + ";" + Duration;

        }

        public string GetGeocodeByAddress(
            string Address, int MapServicesOption,
            string MapServicesKey, string MapServicesLang){

            string Lang, Key, xURL, Str, _Str1, _Str2, Latitude, Longitude;
            int NumPos1, NumPos2;

            string[] Text = new string[100];
            Text[1] = "ERROR 01";   // Google Maps, 'OVER_QUERY_LIMIT', 'You have exceeded your rate-limit for this API.';
            Text[2] = "ERROR 02";   // Google Maps, 'INVALID_REQUEST';
            Text[3] = "ERROR 03";   // Google Maps, 'REQUEST_DENIED', 'The provided API key is invalid.';

            Text[21] = "ERROR 21";   // Bing Maps, 'Not Found', 'One or more locations specified in the waypoint parameter are invalid or require more information.';

            
            switch (MapServicesLang) {
                case "ENU": Lang = "en-us"; break;
                case "RUS": Lang = "ru-ru"; break;
                default: Lang = "en-us"; break;
            };

            if (MapServicesKey != "") Key = "&key=" + MapServicesKey;
            else Key = "";

            xURL = "";
            switch (MapServicesOption) {
                case 0: // "Google Maps"
                    xURL = "https://maps.googleapis.com/maps/api/geocode/xml?address=" +
                            Address + "&language=" + Lang.Substring(0,2) + Key;
                    break;
                case 1: // "Microsoft Bing Maps"
                    xURL = "http://dev.virtualearth.net/REST/v1/Locations?q=" +
                           Address + "&o=xml&c=" + Lang + Key;
                    break;
            };

            WebRequest request = WebRequest.Create(xURL);
            ((HttpWebRequest)request).UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)";
            WebResponse response = request.GetResponse();
            Stream data = response.GetResponseStream();
            StreamReader reader = new StreamReader(data);
            Str = reader.ReadToEnd();
            reader.Close();
            response.Close();

            switch (MapServicesOption) {
                case 0: //"Google Maps"
                    if (Str.IndexOf("OVER_QUERY_LIMIT") > 0) return Text[1] + ": " + Address;
                    if (Str.IndexOf("INVALID_REQUEST") > 0) return Text[2] + ": " + Address;
                    if (Str.IndexOf("REQUEST_DENIED") > 0) return Text[3] + ": " + Address;
                    
                    NumPos1 = Str.IndexOf("<location>") + 10;
                    _Str1 = Str.Substring(NumPos1);
                    NumPos1 = _Str1.IndexOf("<lat>") + 5;
                    NumPos2 = _Str1.IndexOf("</lat>");
                    _Str1 = _Str1.Substring(NumPos1, NumPos2 - NumPos1);
                    _Str1 = _Str1.Replace('.', ',');

                    NumPos1 = Str.IndexOf("<location>") + 10;
                    _Str2 = Str.Substring(NumPos1);
                    NumPos1 = _Str2.IndexOf("<lng>") + 5;
                    NumPos2 = _Str2.IndexOf("</lng>");
                    _Str2 = _Str2.Substring(NumPos1, NumPos2 - NumPos1);
                    _Str2 = _Str2.Replace('.', ',');
                    break;
                case 1: // "Microsoft Bing Maps"
                    if (Str.IndexOf("Not Found") > 0) return Text[21] + ": " + Address;
                    NumPos1 = Str.IndexOf("<Latitude>") + 10;
                    NumPos2 = Str.IndexOf("</Latitude>");
                    _Str1 = Str.Substring(NumPos1, NumPos2 - NumPos1);
                    _Str1 = _Str1.Replace('.', ',');
                    
                    NumPos1 = Str.IndexOf("<Longitude>") + 11;
                    NumPos2 = Str.IndexOf("</Longitude>");
                    _Str2 = Str.Substring(NumPos1, NumPos2 - NumPos1);
                    _Str2 = _Str2.Replace('.', ',');
                    break;
                default:
                    _Str1 = "";
                    _Str2 = "";
                    break;
            }

            Latitude = _Str1;
            Longitude = _Str2;
            return Latitude + ";" + Longitude;

        }

        public bool RunPlanner(string FullFileName) {
            // Запуск системы планирования ;
            Process vrp = new Process();
            vrp.StartInfo.UseShellExecute = false;
            vrp.StartInfo.WorkingDirectory = Path.GetDirectoryName(FullFileName);
            vrp.StartInfo.FileName = FullFileName;
            vrp.StartInfo.Arguments = "";
            vrp.StartInfo.CreateNoWindow = true;
            vrp.Start();
            vrp.WaitForExit();
            vrp.Close();
            return (true);
            // ;
        }
    }
}